#!/bin/bash

# Define variables
PANEL_NAME="AffPanel"
PANEL_DIR="/opt/affpanel"
PANEL_PORT="3333"
APACHE_CONF="/etc/apache2/sites-available/affpanel.conf"
DB_FILE="$PANEL_DIR/users.db"
OS=$(awk -F= '/^ID=/{gsub(/"/, "", $2); print $2}' /etc/os-release)

# Function to install required packages
install_dependencies() {
    echo "📦 Installing Apache, PHP, and SQLite..."
    apt update -y
    apt install -y apache2 php-cli php-fpm php-sqlite3 sqlite3 unzip libapache2-mod-php
}

# Function to ask for admin username & password
setup_authentication() {
    echo "🔑 Setting up authentication for AffPanel..."
    
    read -p "Enter admin username: " ADMIN_USER
    read -s -p "Enter admin password: " ADMIN_PASS
    echo ""
    
    # Store username & hashed password in SQLite
    mkdir -p "$PANEL_DIR"
    sqlite3 "$DB_FILE" "CREATE TABLE IF NOT EXISTS users (id INTEGER PRIMARY KEY, username TEXT, password TEXT);"
    HASHED_PASS=$(php -r "echo password_hash('$ADMIN_PASS', PASSWORD_BCRYPT);")
    sqlite3 "$DB_FILE" "INSERT INTO users (username, password) VALUES ('$ADMIN_USER', '$HASHED_PASS');"
}

# Function to create AffPanel authentication system
setup_panel() {
    echo "📝 Setting up $PANEL_NAME..."
    mkdir -p "$PANEL_DIR"

    # Create login.php
    cat <<EOF > "$PANEL_DIR/login.php"
<?php
session_start();
\$db = new SQLite3('$DB_FILE');

if (\$_SERVER["REQUEST_METHOD"] == "POST") {
    \$username = \$_POST["username"];
    \$password = \$_POST["password"];

    \$stmt = \$db->prepare("SELECT password FROM users WHERE username = :username");
    \$stmt->bindValue(':username', \$username, SQLITE3_TEXT);
    \$result = \$stmt->execute()->fetchArray();

    if (\$result && password_verify(\$password, \$result['password'])) {
        \$_SESSION["authenticated"] = true;
        header("Location: index.php");
        exit();
    } else {
        \$error = "Invalid login credentials!";
    }
}
?>
<!DOCTYPE html>
<html>
<head>
    <title>AffPanel Login</title>
</head>
<body>
    <h2>Login to AffPanel</h2>
    <?php if (isset(\$error)) { echo "<p style='color:red;'>\$error</p>"; } ?>
    <form method="post">
        <label>Username:</label>
        <input type="text" name="username" required><br>
        <label>Password:</label>
        <input type="password" name="password" required><br>
        <input type="submit" value="Login">
    </form>
</body>
</html>
EOF

    # Create index.php (protected by session authentication)
    cat <<EOF > "$PANEL_DIR/index.php"
<?php
session_start();
if (!isset(\$_SESSION["authenticated"])) {
    header("Location: login.php");
    exit();
}
?>
<!DOCTYPE html>
<html>
<head>
    <title>AffPanel</title>
</head>
<body>
    <h2>Welcome to AffPanel</h2>
    <p><a href="logout.php">Logout</a></p>
</body>
</html>
EOF

    # Create logout.php
    cat <<EOF > "$PANEL_DIR/logout.php"
<?php
session_start();
session_destroy();
header("Location: login.php");
exit();
?>
EOF

    chown -R www-data:www-data "$PANEL_DIR"
    chmod -R 755 "$PANEL_DIR"
}

# Function to configure Apache for the panel
configure_apache() {
    echo "🔧 Configuring Apache for $PANEL_NAME..."

    # Ensure Apache listens on port 3333
    grep -qxF "Listen 3333" /etc/apache2/ports.conf || echo "Listen 3333" >> /etc/apache2/ports.conf

    # Create Virtual Host configuration
    cat <<EOF > "$APACHE_CONF"
<VirtualHost *:$PANEL_PORT>
    ServerAdmin admin@localhost
    DocumentRoot "$PANEL_DIR"
    ErrorLog ${APACHE_LOG_DIR}/affpanel-error.log
    CustomLog ${APACHE_LOG_DIR}/affpanel-access.log combined

    <Directory "$PANEL_DIR">
        AllowOverride All
        Require all granted
    </Directory>
</VirtualHost>
EOF

    a2enmod php8.3
    a2ensite affpanel.conf
    systemctl restart apache2
}

# Function to configure the firewall
configure_firewall() {
    echo "🔒 Configuring firewall for $PANEL_NAME..."
    ufw allow "$PANEL_PORT"/tcp
}

# Function to start services
start_services() {
    echo "🚀 Starting services for $PANEL_NAME..."
    systemctl enable --now apache2
    systemctl restart apache2
}

# Ensure script is in Unix format (if run from Windows)
convert_to_unix() {
    echo "🔄 Checking file format..."
    sed -i 's/\r$//' "$0"
}

# Run all functions
convert_to_unix
install_dependencies
setup_authentication
setup_panel
configure_apache
configure_firewall
start_services

echo "✅ Installation complete! Access $PANEL_NAME at http://localhost:$PANEL_PORT"
